/*!
 * @file  Fahrzeug.h
 * @brief Definition der Klasse Fahrzeug.
 */

#ifndef __FAHRZEUG_
#define __FAHRZEUG_

#include <iostream>
using namespace std;

#include "ListenDaten.h"
#include "Koordinaten.h"
class Spur;
#include "Spur.h"

/*!
 * @class Fahrzeug
 * @brief Bewegung auf der Strasze und mit Hilfe der Sensoren mittels Aktoren auf bestimmte Situationen reagieren 
 */
class Fahrzeug : public ListenDaten
{
	public:
		Fahrzeug( );
		Fahrzeug(	int typ, 
					double laenge, 
					Koordinaten farbe, 
					double beschleunigungPositiv, 
					double beschleunigungNegativ, 
					double hoechstgeschwindigkeit, 
					Spur *spur,
					double lebensdauer,
					long int id = 0 );
		
		void takt( double zeitintervall );
		void fahren( double zeitintervall );
		void setzeFahrtenbuch(int *fahrtenbuch, int anzahlWegpunkte);
		
		double gibPosition( );
		Spur *gibSpur( );
		Koordinaten gibFarbe( );
		double gibLaenge( );
		int gibTyp();
		void aufraeumen();
		int gibNaechsteRichtung();
		void exportieren( string cvsDateiname );

		bool istBremsend();
		bool sollEntferntWerden();

		void inString( );
		friend ostream &operator<< ( ostream &os, Fahrzeug *fahrzeug );

	private:
		void clear();
		double gibAbstandZumVordermann( );
		double gibAbstandZurKreuzung();
		void beschleunigePositiv( double zeitintervall );
		void beschleunigeNegativ( double zeitintervall, double intensivitaet );
		void beschleunigeNicht( double zeitintervall );
		void wechselSpur( Spur *spur = 0 );
		double gibBremsweg( );

		double beschleunigungPositiv_;		//!< maximale Beschleunigung des Fahrzeuges in m/s^2
		double beschleunigungNegativ_;		//!< maximale Bremswirkunk in m/s^2
		double hoechstgeschwindigkeit_;		//!< maximale Geschwindigkeit des Fahrzeuges in km/h
		
		double geschwindigkeit_;			//!< aktuelle Geschwindigkeit des Fahrzeuges in km/h
		double kilometerstand_;				//!< vom Fahrzeug zurueckgelegte Entfernung
		double alter_;						//!< gibt die Zeit in s an, die das Fahrzeug seit seiner Entstehung unterwegs ist
		double aktuellePosition_;			//!< aktuelle Position des Fahrzeuges auf der Spur in m gemessen ab dem Begin der Spur
		Spur *aktuelleSpur_;				//!< aktuell befahrene Spur des Fahrzeuges
		Spur *naechsteSpur_;				//!< gibt die als naechstes zu befahrende Spur an.
		int naechsteRichtung_;				//!< gibt an, in welcher Richtung die naechste Spur liegt.
		double lebensdauer_;				//!< Gibt an, wie lange das Fahrzeug noch existieren wird ( -1 = unendlich | 1..n Kreuzungen )
		int *fahrtenbuch_;					//!< vorbestimmte Route des Fahrzeuges ( NULL = keine Route )
		int anzahlWegpunkte_;				//!< gibt die Anzahl der Wegpunkte im Fahrtenbuch an
		int aktuellerWegpunkt_;				//!< gibt an, auf welchem Wegpunkt ein Fahrzeug sich zur Zeit befindet
		
		int typ_;							//!< der Typ des Fahrzeuges ( 1 = Motorrad | 2 = Pkw | 3 = Lkw | 4 = Bus )
		Koordinaten farbe_;					//!< Farbe des Fahrzeuges als rot | gruen | blau
		double laenge_;						//!< Laenge des Fahrzeuges in m

		int bremst_;						//!< Flag, welches anzeigt, ob ein Fahrzeug bremst oder nicht
};

#endif