/**
 * @file  Fehler.cpp
 * @brief Implementierung der Klasse Fehler.
 */


#include <string>
#include <fstream>
#include <time.h>
using namespace std;

#include "Fehler.h"

/*!
 * @fn Fehler::Fehler()
 * @brief Standarkonstruktor fuer die Fehlerklasse.
 * Es wird ein allgemeiner Fehler mit der Fehlernummer 0 generiert.
 *
 * @author Nico Schroeder
 */
Fehler::Fehler(){
    fehlerNummer_ = 0;
    fehlerMeldung_ = "Ein allgemeiner Fehler ist aufgetreten.";
    log();
}

/*!
 * @fn Fehler::Fehler( string FehlerMeldung, int FehlerNummer )
 * @brief Weiterer Konstruktor fuer die Fehlerklasse.
 *
 * @author Nico Schroeder
 *
 * @param fehlerMeldung Diese Fehlermeldung beschreibt das aufgetretene Problem.
 * @param fehlerNummer Dieser optionale Parameter kann eine Fehlernummer beinhalten.
 */
Fehler::Fehler( string fehlerMeldung, int fehlerNummer ){
    fehlerNummer_ = fehlerNummer;
    fehlerMeldung_ = fehlerMeldung;
    log();
}

/*!
 * @fn Fehler &Fehler::operator= ( Fehler &copyFehler )
 * @brief Ueberladener Kopierkonstruktor
 *
 * @author Nico Schroeder
 *
 * @param copyFehler Der zu kopierende Fehler.
 * @return Den neuen Fehler.
 */
Fehler &Fehler::operator= ( Fehler &copyFehler ){
    fehlerMeldung_ = copyFehler.gibMeldung();
    fehlerNummer_ = copyFehler.gibFehlerNummer();
    return *this;
}

/*!
 * @fn string Fehler::gibMeldung()
 * @brief Gibt die den Fehler beschreibende Meldung zurueck.
 *
 * @author Nico Schroeder
 *
 * @return Die Fehlermeldung als string.
 */
string Fehler::gibMeldung(){
    return fehlerMeldung_;
}

/*!
 * @fn int Fehler::gibFehlerNummer()
 * @brief Gibt die den Fehler beschreibende Nummer zurueck.
 *
 * @author Nico Schroeder
 *
 * @return Die Fehlernummer als int.
 */
int Fehler::gibFehlerNummer(){
    return fehlerNummer_;
}

/*!
 * @fn void Fehler::ergaenzeMeldung( string add )
 * @brief Haengt an die bestehende Fehlermeldung etwas an.
 *
 * @author Nico Schroeder
 *
 * @param ergaenzung Der an die Fehlermeldung anzuhaengende Teil.
 */
void Fehler::ergaenzeMeldung( string ergaenzung ){
    fehlerMeldung_ += "\n\n" + ergaenzung;
}

/*!
 * @fn Fehler::log()
 * @brief Schreibt den Fehler in die Datei error.log
 *
 * @author Nico Schroeder
 */
void Fehler::log(){
    ofstream errorLog;
	errorLog.open( "error.log", ios::app );
	errorLog << *this;
}

/*!
 * @fn		ostream &operator<< ( ostream &os, Fehler fehler )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe eines Fehlers ueber cout
 *
 * @author	Nico Schroeder
 *
 * @param	os Der OutStream, in den eingefuegt werden soll.
 * @param	fehler Der auszugebenden Fehler.
 */
ostream &operator<< ( ostream &os, Fehler fehler ){
    // hole die aktuelle Zeit und das Datum
    time_t updatingTime = time( NULL );
    tm *brokenUpdatingTime = localtime( &updatingTime );

    // packe alles in die Datei
    os << endl;
    os << "=======================FEHLER===================================================" << endl;
    os << brokenUpdatingTime->tm_year + 1900 << "-";
    os << brokenUpdatingTime->tm_mon + 1 << " -";
    os << brokenUpdatingTime->tm_mday << "  ";
    os << brokenUpdatingTime->tm_hour << ":";
    os << brokenUpdatingTime->tm_min;
    os << endl;
    os << "--------------------------------------------------------------------------------" << endl;
	os << fehler.fehlerMeldung_ << endl;

	return os;
}