/**
 * @file  Generator.cpp
 * @brief Implementierung der Klasse Generator.
 */

#include "Generator.h"


/*!
 * @fn		Generator::Generator( )
 * @brief	Default-Konstruktor der Klasse Generator
 *
 * @author	Nico Schroeder
 */
Generator::Generator(){
	clear();
	spur_ = 0;
}


/*!
 * @fn		Generator::Generator( Spur* spur, long int id )
 * @brief	Default-Konstruktor der Klasse Generator
 *
 * @author	Nico Schroeder
 * 
 * @param	spur Auf dieser Spur werden die Fahrzeuge erstellt
 * @param	id Die ID des Generators
 */
Generator::Generator( Spur* spur, long int id ){
	clear();
	spur_ = spur;
	idBenutzer_ = id;
}

/*!
 * @fn		void Generator::clear()
 * @brief	Setzt die Einstellungen des Generators auf Defaultwerte
 *
 * @author	Nico Schroeder
 */
void Generator::clear(){
	letzteErstellungszeit_ = 0;
	aktuelleAnzahlFahrzeuge_ = 0;
	anzahlFahrzeuge_ = 0;
	loescheMich_ = 0;
	zufallsFlag_ = 1;
	idBenutzer_ = 0;
	
	fahrzeugTyp_ = 2;
	fahrzeugBeschleunigungPositiv_ = 3;
	fahrzeugBeschleunigungNegativ_ = 7.5;
	fahrzeugLebensdauer_ = -1;
	fahrzeugLaenge_ = 3.5;
	fahrzeugHoechstgeschwindigkeit_ = 200;
	fahrzeugFahrtenbuch_ = 0;
	fahrzeugAnzahlWegpunkte_ = 0;
}

/*!
 * @fn		void Generator::takt(double zeitintervall)
 * @brief	Gibt dem Generator den Takt zum Arbeiten
 *
 * @author	Nico Schroeder
 * 
 * @param	zeitintervall Der Zeitraum seit dem letzten Takt
 * @throw	Fehler* Einen Fehler.
 */
void Generator::takt(double zeitintervall){
	letzteErstellungszeit_ = letzteErstellungszeit_ + zeitintervall;
	if( letzteErstellungszeit_ < taktZeit_ ){
		// tue nichts .. nur die Zeit zaehlen
	}
	else{
		if( aktuelleAnzahlFahrzeuge_ >= anzahlFahrzeuge_ ){
			loescheMich_ = 1;
			return;
		}
		// erstelle ein Fahrzeug und setze de Zeit auf 0
		if( zufallsFlag_ == 1 ){
			zufall();
		}
		try{
			erstelleFahrzeug();
		}
		catch( Fehler *fehler ){
			throw( fehler );
		}
		letzteErstellungszeit_ = 0;
		aktuelleAnzahlFahrzeuge_ ++;
	}
}

/*!
 * @fn		void Generator::aufraeumen()
 * @brief	Muss gestartet werden, bevor ein Generator entfernt wird.
 *
 * @author	Nico Schroeder
 */
void Generator::aufraeumen(){
}


/*!
 * @fn		int Generator::zufallszahl( int minimum, int maximum )
 * @brief	Erzeugt eine Zufallszahl aus dem angegebenen Bereich
 *
 * @author	Nico Schroeder
 *
 * @param	minimum Die kleinste Zahl, die zurueckgegeben werden sollte
 * @param	maximum Die groesste Zahl, die zurueckgegeben werden sollte
 * @return	Die ermittelte Zufallszahl
 * @throw	Fehler* Einen Fehler.
 */
int Generator::zufallszahl( int minimum, int maximum ){
	srand((unsigned)time(NULL));
	int zahl(0);

	if( minimum >= maximum ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDer Zufallsgenerator kann nur mit einem Minimum kleiner als dem Maximum einen sinnvolen Wert ermitteln.");
		throw(fehler);
	}

	do{
		zahl = rand()%(maximum+1-minimum);
		zahl += minimum;
	}while( zahl < minimum || zahl > maximum );

	return zahl;
}

/*!
 * @fn		void Generator::wegpunktHinzufuegen( int richtung )
 * @brief	Fuegt dem zukuenftigen Fahrtenbuch der Fahrzeuge einen neuen Wegpunkt hinzu
 *
 * @author	Nico Schroeder
 *
 * @param	richtung Die Richtung des neuen Wegpunktes
 */
void Generator::wegpunktHinzufuegen( int richtung ){
	int *tmp(0);
	int i(0);
	// wenn es der erste Punkt ist
	if( fahrzeugFahrtenbuch_ == 0 ){
		fahrzeugFahrtenbuch_ = new int[1];
		fahrzeugFahrtenbuch_[0] = richtung;
		fahrzeugAnzahlWegpunkte_ = 1;
	}
	// sonst muessen die Wegpunkte neu sortiert werden
	else{
		tmp = fahrzeugFahrtenbuch_;
		fahrzeugFahrtenbuch_ = 0;
		// neues Array anlegen, welches um eine Einheit groesser ist
		fahrzeugFahrtenbuch_ = new int[(fahrzeugAnzahlWegpunkte_+1)];
		// die vorhandenen Eintraege hinueberkopieren
		for( i = 0; i < fahrzeugAnzahlWegpunkte_; i++ ){
			fahrzeugFahrtenbuch_[i] = tmp[i];
		}
		// den neuen Eintrag hunzufuegen
		fahrzeugAnzahlWegpunkte_++;
		fahrzeugFahrtenbuch_[fahrzeugAnzahlWegpunkte_-1] = richtung;
		// das alte Fahrtenbuch loeschen
		delete tmp;
	}
}

/*!
 * @fn		void Generator::setzeAnzahlFahrzeuge(int anzahl)
 * @brief	Setzt die Anzahl der zu erstellenden Fahrzeuge und startet den Zaehler neu
 *
 * @author	Nico Schroeder
 * 
 * @param	anzahl Die Anzahl der noch zu erstellenden Fahrzeuge
 */
void Generator::setzeAnzahlFahrzeuge(int anzahl){
	anzahlFahrzeuge_ = anzahl;
	aktuelleAnzahlFahrzeuge_ = 0;
}

/*!
 * @fn		void Generator::setzeFahrzeugeigenschaften(int typ, double beschleunigungPositiv, double beschleunigungNegativ, int lebensdauer, double laenge, Koordinaten farbe, double hoechstgeschwindigkeit )
 * @brief	Setzt die Eigenschaften der Fahrzeuge, welche erstellt werden. Die Zufallsmodus wird ausgeschaltet.
 *
 * @author	Nico Schroeder
 * 
 * @param	typ Der Typ der zu erstellenden Fahrzeuge (BIKE|CAR|LKW|BUS)
 * @param	beschleunigungPositiv Die Beschleunigung der zu erstellenden Fahrzeuge
 * @param	beschleunigungNegativ Die maximale Verzoegerung der zu erstellenden Fahrzeuge
 * @param	lebensdauer Die Anzahl der zu ueberfahrenden Kreuzungen eines Fahrzeuges
 * @param	laenge Die Laenge der Fahrzeuge in m.
 * @param	farbe Die Farbe als RGB-Wert
 * @param	hoechstgeschwindigkeit
 * @throw	Fehler* Einen Fehler.
 */
void Generator::setzeFahrzeugeigenschaften(int typ, double beschleunigungPositiv, double beschleunigungNegativ, int lebensdauer, double laenge, Koordinaten farbe, double hoechstgeschwindigkeit ){
	if( typ < 1 || typ > 4 ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDer Typ fuer Fahrzeuge in einem Generator entspricht nicht den geforderten Parametern.");
		throw(fehler);
	}
	fahrzeugTyp_ = typ;
	fahrzeugBeschleunigungPositiv_ = beschleunigungPositiv;
	fahrzeugBeschleunigungNegativ_ = beschleunigungNegativ;
	fahrzeugLebensdauer_ = lebensdauer;
	fahrzeugFarbe_ = farbe;
	fahrzeugLaenge_ = laenge;
	fahrzeugHoechstgeschwindigkeit_ = hoechstgeschwindigkeit;
	// Nun nicht mehr zufaellig waehlen, sondern die Voreinstellungen nutzen
	zufallsFlag_ = 0;
}

/*!
 * @fn		void Generator::setzeTaktZeit( double zeitintervall )
 * @brief	Setzt die Eigenschaften der Fahrzeuge, welche erstellt werden. Die Zufallsmodus wird ausgeschaltet.
 *
 * @author	Nico Schroeder
 * 
 * @param	zeitintervall Die Zeit in Sekunden nach der ein neues Fahrzeug erstellt werden soll
 */
void Generator::setzeTaktZeit( double zeitintervall ){
	taktZeit_ = zeitintervall;
}

/*!
 * @fn		long int Generator::gibId()
 * @brief	Liefert die ID des Generators zurueck
 *
 * @author	Nico Schroeder
 * 
 * @return	Die ID
 */
long int Generator::gibId(){
	return id_;
}


/*!
 * @fn		int Generator::gibAnzahlFahrzeuge()
 * @brief	Gibt die Anzahl der maximal zu erstellenden Fahrzeuge zurueck
 *
 * @author	Nico Schroeder
 * 
 * @return	Die Anzahl der Fahrzeuge
 */
int Generator::gibAnzahlFahrzeuge(){
	return anzahlFahrzeuge_;
}

/*!
 * @fn		int Generator::gibAktuelleAnzahlFahrzeuge()
 * @brief	Gibt die Anzahl der schon erstellten Fahrzeuge zurueck
 *
 * @author	Nico Schroeder
 * 
 * @return	Die Anzahl der Fahrzeuge
 */
int Generator::gibAktuelleAnzahlFahrzeuge(){
	return aktuelleAnzahlFahrzeuge_;
}

/*!
 * @fn		bool Generator::sollEntferntWerden()
 * @brief	Ermittelt, ob ein Generator aus dem Universum genommen werden soll.
 *
 * @author	Nico Schroeder
 *
 * @return	TRUE, wenn ein Generator enfernt werden soll, sonst FALSE
 */
bool Generator::sollEntferntWerden(){
	return loescheMich_;
}

/*!
 * @fn		void Generator::zufall()
 * @brief	Fuellt die Fahrzeugeigenschaften mit zufaellig ausgewaehlten Werten
 *
 * @author	Nico Schroeder
 */
void Generator::zufall(){
	int rgb(0), farbe(0);
	
	// Zu erst den Fahrzeugtyp ermitteln
	fahrzeugTyp_ = zufallszahl( 1, 4 );
	// Auf Basis des Types Vmax, a_pos, a_neg, laenge bestimmen
	switch( fahrzeugTyp_ ){
		case BIKE:{
			fahrzeugHoechstgeschwindigkeit_ = zufallszahl( 200, 360 );
			fahrzeugBeschleunigungPositiv_ = (double) zufallszahl( 70, 93 ) / 10;
			fahrzeugBeschleunigungNegativ_ = (double) zufallszahl( 70, 100 ) / -10;
			fahrzeugLaenge_ = (double) zufallszahl( 13, 25 ) / 10;
			break;
			   }
		case PKW:{
			fahrzeugHoechstgeschwindigkeit_ = zufallszahl( 170, 250 );
			fahrzeugBeschleunigungPositiv_ = (double) zufallszahl( 19, 35 ) / 10;
			fahrzeugBeschleunigungNegativ_ = (double) zufallszahl( 60, 75 ) / -10;
			fahrzeugLaenge_ = (double) zufallszahl( 25, 50 ) / 10;
			break;
			   }
		case LKW:{
			fahrzeugHoechstgeschwindigkeit_ = zufallszahl( 70, 90 );
			fahrzeugBeschleunigungPositiv_ = (double) zufallszahl( 13, 14 ) / 10;
			fahrzeugBeschleunigungNegativ_ = (double) zufallszahl( 45, 55 ) / -10;
			fahrzeugLaenge_ = (double) zufallszahl( 6, 12 );
			break;
			   }
		case BUS:{
			fahrzeugHoechstgeschwindigkeit_ = zufallszahl( 80, 130 );
			fahrzeugBeschleunigungPositiv_ = (double) zufallszahl( 15, 16 ) / 10;
			fahrzeugBeschleunigungNegativ_ = (double) zufallszahl( 45, 55 ) / -10;
			fahrzeugLaenge_ = (double) zufallszahl( 6, 10 );
			break;
			   }
	}
	// Die Farbe bestimmen
	rgb = zufallszahl( 0, 2 );
	if( rgb == 0 ){
		fahrzeugFarbe_.x = zufallszahl( 150, 255 );
		fahrzeugFarbe_.y = 0;
		fahrzeugFarbe_.z = 0;
	}
	else if( rgb == 1 ){
		fahrzeugFarbe_.x = 0;
		fahrzeugFarbe_.y = zufallszahl( 150, 255 );
		fahrzeugFarbe_.z = 0;
	}
	else{
		fahrzeugFarbe_.x = 0;
		fahrzeugFarbe_.y = 0;
		fahrzeugFarbe_.z = zufallszahl( 150, 255 );
	}

	fahrzeugLebensdauer_ = zufallszahl( -1, 1000 );
}

/*!
 * @fn		void Generator::erstelleFahrzeug()
 * @brief	Erstellt ein neues Fahrzeug an der Stelle spur und fuegt dieses in die Fahrzeugliste ein
 *
 * @author	Nico Schroeder
 * @throw	Fehler* Einen Fehler.
 */
void Generator::erstelleFahrzeug(){
	Fahrzeug *fahrzeug;
	try{
		fahrzeug = new Fahrzeug( fahrzeugTyp_, fahrzeugLaenge_, fahrzeugFarbe_, 
									fahrzeugBeschleunigungPositiv_, fahrzeugBeschleunigungNegativ_, 
									fahrzeugHoechstgeschwindigkeit_, spur_, fahrzeugLebensdauer_ );
		if( fahrzeugFahrtenbuch_ != 0 ){
			fahrzeug->setzeFahrtenbuch( fahrzeugFahrtenbuch_, fahrzeugAnzahlWegpunkte_ );
		}
		fahrzeugliste_->datenHinzufuegen( fahrzeug );
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		void Generator::setzeFahrzeugliste( Liste *fahrzeugliste )
 * @brief	Setzt einen Zeiger auf die Fahrzeugliste in die spaeter Fahrzeuge eingefuegt werden sollen
 *
 * @author	Nico Schroeder
 *
 * @param	fahrzeugliste Die neue Fahrzeugliste
 */
void Generator::setzeFahrzeugliste( Liste *fahrzeugliste ){
	fahrzeugliste_ = fahrzeugliste;
}


/*
 * @fn		void Generator::inString( )
 * @brief	Gibt den Generator auf der Konsole aus
 *
 * @author	Nico Schroeder
 */
void Generator::inString( ){
	cout << this;
}

/*!
 * @fn		ostream &operator<< ( ostream &os, Generator *generator )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe eines Fahrzeuges ueber cout
 *
 * @author	Nico Schroeder
 * @param	os Den zu modifizierenden OutStream.
 * @param	generator Den auszugebenden Generator.
 * @return	Den modifizierten OutStream.
 */
ostream &operator<< ( ostream &os, Generator *generator ){
	os << "----------Generator----------" << endl;
	os << "ID:                   " << generator->id_ << endl;
	os << "Erstellzeit:          " << generator->taktZeit_ << endl;
	os << "noch erstllende Fzg:  " << generator->anzahlFahrzeuge_ - generator->aktuelleAnzahlFahrzeuge_ << endl;
	
	return os;
}

/*!
 * @fn		void Generator::exportieren( string cvsDateiname )
 * @brief	Exportiert die aktuellen Generatordaten in ein CSV-File. Zur Zeit ist diese Funktion nicht implementiert, da sie nicht benoetigt wird.
 *
 * @author	Nico Schroeder
 *
* @param	cvsDateiname Der Pfad uns Name zu der abzuspeichernden CSV-Datei
 */
void Generator::exportieren( string cvsDateiname ){
	// Datei oeffnen
	// Daten ausstreamen
	// Datei schliessen
}