/**
 * @file  Generator.h
 * @brief Definition der Klasse Generator.
 */

#ifndef __GENERATOR_
#define __GENERATOR_

#include <time.h> 
#include <stdlib.h>

#include "Fahrzeug.h"
#include "Fehler.h"
#include "Konstanten.h"
#include "Liste.h"

/*!
 * @class Generator
 * @brief Die Klasse Generator kann selbststaendig auf eine bestimmte Spur im Universum Fahrzeuge erstellen.
 */
class Generator : public ListenDaten
{
	public:
		Generator( );
		Generator(Spur *spur, long int id);

		void takt(double zeitintervall);

		void setzeAnzahlFahrzeuge(int anzahl);
		void setzeFahrzeugeigenschaften(int typ, double beschleunigungPositiv, double beschleunigungNegativ, int lebensdauer, double laenge, Koordinaten farbe, double hoechstgeschwindigkeit );
		void wegpunktHinzufuegen( int richtung );
		void setzeTaktZeit( double zeitintervall );
		void setzeFahrzeugliste( Liste *fahrzeugliste );

		long int gibId();
		int gibAnzahlFahrzeuge();
		int gibAktuelleAnzahlFahrzeuge();
		bool sollEntferntWerden();
		void aufraeumen();

		void inString( );
		void exportieren( string cvsDateiname );
		friend ostream &operator<< ( ostream &os, Generator *generator );

	private:
		void zufall();
		void erstelleFahrzeug();
		void clear();
		int zufallszahl( int minimum, int maximum );

		int anzahlFahrzeuge_;				//!< Die Anzahl der zu generierenden Fahrzeuge
		int aktuelleAnzahlFahrzeuge_;		//!< Die Anzahl der schon generierden Fahrzeuge
		Spur *spur_;						//!< Die Spur, auf welcher die Fahrzeuge erstellt werden sollen
		Liste *fahrzeugliste_;				//!< Die Liste von Fahrzeugen, in welches die Fahrzeuge eingefuegt werden sollen
		bool zufallsFlag_;					//!< Gibt an, ob ein zufaelliges Fahrzeug generiert werden soll oder die Einstellungen beibehalten werden sollen
		double taktZeit_;					//!< Zeit in Sekunden, nach der ein Fahrzeug erstellt werden soll
		double letzteErstellungszeit_;		//!< Zeit seit der letzten Erstellung eines Fahrzeuges
		
		int fahrzeugTyp_;					//!< Dieser Fahrzeugtyp soll erstellt werden
		double fahrzeugBeschleunigungPositiv_;	//!< Diese Beschleunigung besitzen die zu erstellenden Fahrzeuge
		double fahrzeugBeschleunigungNegativ_;	//!< Maximale Bremskraft eines zu erstellenden Fahrzeuges
		int fahrzeugLebensdauer_;			//!< Die Lebensdauer der Fahrezeuge
		double fahrzeugLaenge_;				//!< Die Laenge der Fahrzeuge
		Koordinaten fahrzeugFarbe_;			//!< Die Farbe der Fahrzeuge
		double fahrzeugHoechstgeschwindigkeit_;	//!< Die maximale Geschwindigkeit der Fahrzeuge
		int *fahrzeugFahrtenbuch_;			//!< Das Fahrtenbuch des Fahrzeuges
		int fahrzeugAnzahlWegpunkte_;		//!< Die Anzahl der im Fahrtenbuch befindlichen Wegpunkte

};

#endif


