/*! 
 * @file  Kreuzung.h
 * @brief Definition der Klasse Kreuzung.
 */

#ifndef __KREUZUNG_
#define __KREUZUNG_

#include "Fehler.h"
#include "ListenDaten.h"
#include "Spur.h"

/*!
 * @class Kreuzung
 * @brief Kreuzungen sind die Schnittpunkte der Spuren. Auf den Kreuzungen stehen auch die Ampeln, welche mit einem Ampelalgortihmus gesteuert werden.
 */
class Kreuzung : public ListenDaten
{
	public:
		Kreuzung( long int id = 0 );
		void takt( double zeitintervall );
		void setzeSpur( int himmelsrichtung, int abbiegerichtung, Spur *spur );
		Spur* gibSpur( int himmelsrichtung, int abbiegerichtung );
		void aufraeumen();
		void setzeAmpelalgorithmus( int id );
		void exportieren( string cvsDateiname );

		friend ostream &operator<< ( ostream &os, Kreuzung *kreuzung );
		void inString( );
				
	private:

		void algorithmusZeitZeit( double zeitintervall );
		void algorithmusZeitMenge( double zeitintervall );
		void algorithmusStopp( double zeitintervall );
		void algorithmusMengeMenge( double zeitintervall );
		void clear();

		Spur* spurenListe_[4][3];			//!< Die an der Kreuzung beteiligten Spuren in einem zweidimensionalen Array [OST|SUED|WEST|NORD][LINKS|MITTE|RECHTS].
		int idAmpelAlgorithmus_;			//!< Die ID des zu benutzenden Ampelalgorithmuses.
		double zeitLetzteSchaltung_[4][3];	//!< Gibt die Zeit an, die seit der letzen Schaltung auf einer bestimmten Spur vergangen ist.
		double zeitLetzteSchaltungGlobal_;	//!< Gibt die Zeit an, die seit der letzen Schaltung auf der gesamten Kreuzung vergangen ist.
		int mengeLetzteSchaltungGlobal_;	//!< Gibt die Menge an Fahrzeugen an, die seit der letzten Schaltung ueber die Ampel gefahren sind.
		int schaltungstakt_;				//!< Wenn ein Algorithmus nach einem bestimmten Takt ablaeuft, so wird hier der Takt gezaehlt.
		double schaltzeit_;					//!< Wenn ein Algortihmus immer nach einer bestimmten Zeit schaltet, so wird dieser Wert hier in s gespeichert.
		int statistik_[4][3][4];			//!< Statistik ueber die Anzahl der Fahrzeuge, die ueber die Kreuzung (eine bestimmte Spur, ein bestimmter Fahrzeugtyp) gefahren sind.
		int schaltreihenfolge_[8];			//!< Wenn ein Algorithmus die Schaltreihenfolge selber festlegen kann, so geschieht das in diesem Array.
		int schaltmenge_[8];				//!< Wenn ein Algorithmus immer nach einer bestimmten Anzahl durchgefahrener Fahrzeuge auf einer Spur schlatet, so wird der Wert hier gespeichert.
};

#endif


