/*!
 * @file  Liste.cpp
 * @brief Impelemtierung der Klassen Liste und ListenObjekt.
 */

#include "Liste.h"

/*!
 * @fn		ListenObjekt::ListenObjekt()
 * @brief	Default-Konstruktor der Klasse ListenObjekt
 *
 * @author	Nico Schroeder
 */
ListenObjekt::ListenObjekt(){
	listenDaten_ = NULL;
}


/*!
 * @fn		ListenObjekt::ListenObjekt(ListenDaten *listenDaten)
 * @brief	Konstruktor der Klasse ListenObjekt
 *
 * @author	Nico Schroeder
 * 
 * @param	listenDaten Der Inhalt fuer das Objekt der Liste
 */
ListenObjekt::ListenObjekt(ListenDaten *listenDaten){
	listenDaten_ = listenDaten;
}


/*!
 * @fn		ListenObjekt *ListenObjekt::gibNext()
 * @brief	Gibt den Nachfolger des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 *
 * @return	Der Nachfolger
 */
ListenObjekt *ListenObjekt::gibNext(){
	return next_;
}


/*!
 * @fn		ListenObjekt *ListenObjekt::gibBefore()
 * @brief	Gibt den Vorgaenger des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 *
 * @return	Der Vorgaenger
 */
ListenObjekt *ListenObjekt::gibBefore(){
	return before_;
}


/*!
 * @fn		void ListenObjekt::setzeNext(ListenObjekt *next)
 * @brief	Setzt den Nachfolger des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 */
void ListenObjekt::setzeNext(ListenObjekt *next){
	next_ = next;
}


/*!
 * @fn		void ListenObjekt::setzeBefore(ListenObjekt *before)
 * @brief	Setzt den Nachfolger des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 */
void ListenObjekt::setzeBefore(ListenObjekt *before){
	before_ = before;
}


/*!
 * @fn		ListenDaten *ListenObjekt::gibDaten()
 * @brief	Gibt den Inhalt des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 *
 * @return	Der Inhalt
 */
ListenDaten *ListenObjekt::gibDaten(){
	return listenDaten_;
}


/*!
 * @fn		void ListenObjekt::setzeDaten(ListenDaten *listenDaten)
 * @brief	Setzt den Inhalt des Objektes aus der Listen
 *
 * @author	Nico Schroeder
 */
void ListenObjekt::setzeDaten(ListenDaten *listenDaten){
	listenDaten_ = listenDaten;
}


/*!
 * @fn		void ListenObjekt::takt(double zeitintervall)
 * @brief	Gibt den Takt an den Inhalt des aktuellen ListenObjektes weiter
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall Die Zeit in Sekunden seit dem letzten Aufruf der Funktion.
 */
void ListenObjekt::takt(double zeitintervall){
	listenDaten_->takt(zeitintervall);
}

/*!
 * @fn		void ListenObjekt::exportieren( string dateiname )
 * @brief	Veranlasst den Inhalt des aktuellen ListenObjektes sich zu exportieren
 *
 * @author	Nico Schroeder
 *
 * @param	dateiname Der Pfad uns Name zu der abzuspeichernden CSV-Datei
 * @throw	Fehler* Einen Fehler.
 */
void ListenObjekt::exportieren( string dateiname ){
	try{
		listenDaten_->exportieren( dateiname );
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}


/*!
 * @fn		ostream &operator<< ( ostream &os, ListenObjekt *listenObjekt )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe eines Listenobjektes ueber cout
 *
 * @author	Nico Schroeder
 * 
 * @param	os Der zu modifizierende OutStream.
 * @param	listenObjekt Das auszugebende ListenObjekt.
 * @return	Der modifizierte OutStream.
 */
ostream &operator<< ( ostream &os, ListenObjekt *listenObjekt ){
	os << listenObjekt->gibDaten();
    return os;
}





/*!
 * @fn		Liste::Liste( )
 * @brief	Default-Konstruktor der Klasse Liste
 *
 * @author	Nico Schroeder
 */
Liste::Liste( ){
	// Das Dummyelement erzeugen
	listenAnfang_ = new ListenObjekt( NULL );
	listenAnfang_->setzeNext(listenAnfang_);
	listenAnfang_->setzeBefore(listenAnfang_);
	aktuelleMaxId_ = 0;
	aktuellesObjekt_ = listenAnfang_;
}

/*!
 * @fn		void Liste::takt(double zeitintervall)
 * @brief	Gibt das Taktsignal an alle eingetragenen Objekte weiter
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall Die Zeit in Sekunden seit dem letzten Aufruf der Funktion.
 * @throw	Fehler* Einen Fehler.
 */
void Liste::takt(double zeitintervall){
	ListenObjekt *tmpObjekt = listenAnfang_;
	ListenDaten *listenDaten(0);
	
	try{
		while( tmpObjekt->gibNext()->gibDaten() != NULL ){
			tmpObjekt = tmpObjekt->gibNext();
			listenDaten = tmpObjekt->gibDaten();
			if(listenDaten->sollEntferntWerden()){
				// tmpObjekt = tmpObjekt->gibBefore();
				listenDaten->aufraeumen();
				// datenEntfernen(listenDaten->gibId());
				// delete listenDaten;
				// belasse die Daten und takte sie nicht mehr an
			}
			else{
				tmpObjekt->takt(zeitintervall);
			}
		}
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		void Liste::exportieren( string dateiname )
 * @brief	Exportiert die gesamte Liste in ein CSV-File
 *
 * @author	Nico Schroeder
 *
 * @param	dateiname Der Pfad und Name zu der abzuspeichernden CSV-Datei
 * @throw	Fehler* Einen Fehler.
 */
void Liste::exportieren( string dateiname ){
	ListenObjekt *tmpObjekt = listenAnfang_;
	ListenDaten *listenDaten(0);

	while( tmpObjekt->gibNext()->gibDaten() != NULL ){
		tmpObjekt = tmpObjekt->gibNext();
		listenDaten = tmpObjekt->gibDaten();
		try{
			tmpObjekt->exportieren( dateiname );
		}
		catch( Fehler *fehler ){
			throw( fehler );
		}
	}
}

/*!
 * @fn		void Liste::datenHinzufuegen(ListenDaten *listenDaten)
 * @brief	Fuegt ein neues Datum in die Liste ein und gibt diesem ein ID
 *
 * @author	Nico Schroeder
 * 
 * @param	listenDaten Das einzutragende Datum
 */
void Liste::datenHinzufuegen(ListenDaten *listenDaten){
	aktuelleMaxId_++;
	listenDaten->setzeId( aktuelleMaxId_ );
	// Ein neues Objekt anlegen
	ListenObjekt *neuesObjekt = new ListenObjekt(listenDaten);
	// Das angelegte Objekt in die Liste einflaechten
	neuesObjekt->setzeNext(listenAnfang_);
	neuesObjekt->setzeBefore(listenAnfang_->gibBefore());
	listenAnfang_->setzeBefore(neuesObjekt);
	neuesObjekt->gibBefore()->setzeNext(neuesObjekt);
}


/*!
 * @fn		void Liste::datenEntfernen(long int id)
 * @brief	Entfernt ein Datum aus der Liste
 *
 * @author	Nico Schroeder
 * 
 * @param	id Die ID des zu loeschenden Datums
 */
void Liste::datenEntfernen(long int id){
	ListenObjekt *tmpObjekt = listenAnfang_;
	while( tmpObjekt->gibNext()->gibDaten() != NULL ){
		tmpObjekt = tmpObjekt->gibNext();
		// wenn das aktuelle Objekt das gesuchte ist
		if( tmpObjekt->gibDaten()->gibId() == id ){
			// Die Zeiger des Vorgaengers und Nachfolgers verbiegen und danach das aktuelle Objekt loeschen
			tmpObjekt->gibBefore()->setzeNext(tmpObjekt->gibNext());
			tmpObjekt->gibNext()->setzeBefore(tmpObjekt->gibBefore());
			delete tmpObjekt;
			return;
		}
	}
}

/*!
 * @fn		void Liste::geheZuErsten()
 * @brief	Setzt den internen Zeiger auf das erste Element (Dummyelement)
 *
 * @author	Nico Schroeder
 */
void Liste::geheZuErsten(){
	aktuellesObjekt_ = listenAnfang_;
}

/*!
 * @fn		void Liste::geheZuNaechsten()
 * @brief	Setzt den internen Zeiger auf das naechste
 *
 * @author	Nico Schroeder
 */
void Liste::geheZuNaechsten(){
	aktuellesObjekt_ = aktuellesObjekt_->gibNext();
}

/*!
 * @fn		bool Liste::istEnde()
 * @brief	Trifft eine Aussage, ob das aktuelle Objekt das letzte in der Liste ist
 *
 * @author	Nico Schroeder
 *
 * @return	TRUE wenn es das letzte ist, sonst FALSE
 */
bool Liste::istEnde(){
	if( aktuellesObjekt_->gibNext()->gibDaten() == 0 )
		return 1;
	else
		return 0;
}

/*!
 * @fn		ListenDaten Liste::gibDaten()
 * @brief	Gibt die Daten des aktuellen Listenobjektes zurueck
 *
 * @author	Nico Schroeder
 *
 * @return	Die Daten
 */
ListenDaten *Liste::gibDaten(){
	return aktuellesObjekt_->gibDaten();
}


/*!
 * @fn		ostream &operator<< ( ostream &os, Liste *liste )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe einer Liste ueber cout
 *
 * @author	Nico Schroeder
 * @param	os Der zu modifizierende OutStream.
 * @param	liste Die auszugebende Liste.
 * @return	Der modifizierte OutStream.
 */
ostream &operator<< ( ostream &os, Liste *liste ){
	ListenObjekt *tmpObjekt = liste->listenAnfang_;
	os << "----------------Liste-----------------" << endl;
	while( tmpObjekt->gibNext()->gibDaten() != NULL ){
		tmpObjekt = tmpObjekt->gibNext();
		os << tmpObjekt;
	}	
	return os;
}