/*!
 * @file  Liste.h
 * @brief Definition der Klassen Liste und ListenObjekt.
 */ 

#ifndef __LISTE_
#define __LISTE_

#include <iostream>
using namespace std;

#include "ListenDaten.h"
#include "Fehler.h"

/*!
 * @class ListenObjekt
 * @brief Hilfsklasse fuer die Liste.
 */ 
class ListenObjekt
{
	public:
		ListenObjekt();
		ListenObjekt(ListenDaten *listenDaten);
		
		ListenObjekt *gibNext();
		ListenObjekt *gibBefore();
		ListenDaten *gibDaten();

		void setzeNext(ListenObjekt *next);
		void setzeBefore(ListenObjekt *before);
		void setzeDaten(ListenDaten *listenDaten);
		
		void takt(double zeitintervall);
		void exportieren( string dateiname );

		friend ostream &operator<< ( ostream &os, ListenObjekt *listenObjekt );

	private:
		ListenDaten *listenDaten_;				//!< Inhalt des Listenobjekte
		ListenObjekt *next_;					//!< Nachfolger des aktuellen Listenobjektes
		ListenObjekt *before_;					//!< Vorgaenger des aktuellen Listenobjektes
};


/*!
 * @class Liste
 * @brief Beinhaltet alle Listenobjekte und kann ihnen den Takt vorgeben.
 * Organisiert wird die Liste als eine doppelt verkettete Liste, bei der das Ende wieder auf den Anfang verweisst.
 * Zur besseren Orientierung wird ein Dummyelement eingefuegt, bei dem gibListenDaten() NULL liefert. Dieses Dummyelement dient
 * als Eingangspunkt fuer die Liste.
 */ 
class Liste
{
	public:
		Liste( );
		
		void datenHinzufuegen(ListenDaten *listenDaten);
		void datenEntfernen(long int id);
		ListenDaten *gibDaten();
		void geheZuErsten();
		void geheZuNaechsten();
		bool istEnde();

		void takt(double zeitintervall);
		void exportieren( string dateiname );

		friend ostream &operator<< ( ostream &os, Liste *liste );

	private:
		ListenObjekt *listenAnfang_;			//!< Anfang der Liste (Ankerpunkt)
		ListenObjekt *aktuellesObjekt_;			//!< Das aktuelle Objekt, welches bei gibDaten() ausgegeben wird
		long int aktuelleMaxId_;				//!< Die aktuell hoechste ID, zur Ermittlung der naechsten ID
};


#endif
