/*! 
 * @file  Spur.h
 * @brief Definition der Klasse Spur.
 */
  
#ifndef __SPUR_
#define __SPUR_

#include <iostream>
using namespace std;

class Fahrzeug;
#include "Fahrzeug.h"
#include "Koordinaten.h"

/*!
 * @class Spur
 * @brief Auf den Spuren bewegen sich die Fahrzeuge.
 */
class Spur
{
	public:
		Spur( );
		Spur(long int id, double laenge, int maximalGeschwindigkeit);
		int gibAnzahlFahrzeuge( );
		int gibAnzahlAbgefahrenerFahrzeuge( int typ = 0 );
		Fahrzeug *gibVorgaenger(Fahrzeug *fahrzeug);
		Fahrzeug *gibLetztes();
		double gibLaenge();
		long int gibId();
		void setzeNaechstesVerbindungselement(Spur *spur, int richtung, double wahrscheinlichkeit);
		double gibWahrscheinlichkeit(int richtung);
		void anmelden(Fahrzeug *fahrzeug);
		void abmelden(Fahrzeug *fahrzeug);
		Spur *gibNaechsteSpur(int richtung);
		double gibMaximaleGeschwindigkeit();
		void clear();
		void setzeGruen();
		void setzeGelb();
		void setzeRot();
		void setzeSackgasse();
		bool istGruen();
		bool istGelb();
		bool istRot();
		bool istSackgasse();
		int gibEndzustand( );
		Koordinaten gibKoordinaten();
		void setzeKoordinaten( Koordinaten koordinaten );
		int gibAusrichtung();
		void setzeAusrichtung( int ausrichtung );
		int gibRichtung();
		void setzeRichtung( int richtung );
		int gibAbbiegerichtung();
		void setzeAbbiegerichtung( int abbiegerichtung );
		long int gibNachfolgerId(int richtung);
		void setzeNaechstesVerbindungselement(int richtung, long int id, double wahrscheinlichkeit);

		friend ostream &operator<< ( ostream &os, Spur *spur );
	private:
		long int id_;					//!< Die ID zum Wiederfinden der Spur
		double laenge_;					//!< Laenge der Spur in m.
		int maximalGeschwindigkeit_;	//!< Gibt die maximal auf der Spur erlaube Hoechstgeschwindigkeit an.
		int endzustand_;				//!< Gibt den Endzustand de Spur an ( 1 = gruen | 2 = gelb | 3 = rot | 4 = sackgasse ).
		Spur *spur_[3];					//!< Zeiger auf bis zu 3 angrenzende Spuren.
		long int nachfolger_[3];		//!< ID von bis zu 3 angrenzenden Spuren
		double wahrscheinlichkeit_[3];	//!< Wahrscheinlichkeit, mit der eine der angrenzenden Spuren befahren wird ( 0 = keine Spur vorhanden | 0.1 ... 0.9 | 1 = nur diese Spur wird befahren ).
		Fahrzeug *fahrzeugListe_[1000];	//!< Liste von Zeigern auf die Fahrzeuge ... erleichtert die Abstandsberechnung ungemein.
		int anzahlFahrzeuge_;			//!< Anzahl der zur Zeit auf der Spur und somit in der Liste fahrzeuge beindlichen Fahrzeuge.
		Koordinaten koordinaten_;		//!< Beinhaltet die Koordinaten der Spur, zur Darstellung auf der GUI.
		int ausrichtung_;				//!< Gibt die Ausrichtung der Spur an.
		int richtung_;					//!< Gibt die Richtung der Spur an.
		int abbiegerichtung_;			//!< Zeigt, in welche Richtung die aktuelle Spur geht.
		int statistik_[4];				//!< Fuehrt eine Statistik darueber, wieviele Fahrzeuge von einem bestimmten Typ von der Spur auf einen der Nachfolger wechseln.

		void setzeEndzustand(int zustand);
		
};

#endif