/*!
 * @file  TraffSpot.cpp
 * @brief Implementierung der Klassen TraffSpot.
 */ 

#include "TraffSpot.h"

/*!
 * @fn		TraffSpot::TraffSpot()
 * @brief	Der Defaultkonstruktor der Klasse TraffSpot
 * 
 * @author	Nico Schroeder
 */
TraffSpot::TraffSpot(){
	fahrzeugliste_ = 0;
	generatorliste_ = 0;
	kreuzungsliste_ = 0;
	spurenliste_ = 0;
}

/*!
 * @fn		void TraffSpot::takt()
 * @brief	Diese Funktion regt das gesamte Universum zum aggieren an. Die Reihenfolge ist folgendermassen: 1. Kreuzungen (Ampeln), 2. Fahrzeuge und 3. Generatoren.
 * 
 * @author	Nico Schroeder
 *
 * @throw	Fehler* Einen Fehler.
 */
void TraffSpot::takt(){
	// Im Universium muessen die Beteiligten aggieren
	// Dies sind die Generatoren, Kreuzungen (Ampeln) und die Fahrzeuge
	vergangeneZeit_ += taktZeit_;
	try{
		kreuzungsliste_->takt( taktZeit_ );
		fahrzeugliste_->takt( taktZeit_ );
		generatorliste_->takt( taktZeit_ );
	}
	catch( Fehler* fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		Liste *TraffSpot::gibFahrzeugliste()
 * @brief	Gibt die aktuelle Liste aller Fahrzeuge zurueck
 * 
 * @author	Nico Schroeder
 *
 * @return	Die Liste der Fahrzeuge im Universum
 */
Liste *TraffSpot::gibFahrzeugliste(){
	return fahrzeugliste_;
}

/*!
 * @fn		Liste *TraffSpot::gibGeneratorliste()
 * @brief	Gibt die aktuelle Liste aller Generatoren zurueck
 * 
 * @author	Nico Schroeder
 *
 * @return	Die Liste der Generatoren im Universum
 */
Liste *TraffSpot::gibGeneratorliste(){
	return generatorliste_;
}

/*!
 * @fn		Liste *TraffSpot::gibKreuzungsliste()
 * @brief	Gibt die aktuelle Liste aller Kreuzungen zurueck
 * 
 * @author	Nico Schroeder
 *
 * @return	Die Liste der Kreuzungen im Universum
 */
Liste *TraffSpot::gibKreuzungsliste(){
	return kreuzungsliste_;
}

/*!
 * @fn		Spur **TraffSpot::gibSpurenliste()
 * @brief	Gibt die aktuelle Liste aller Spuren zurueck
 * 
 * @author	Nico Schroeder
 *
 * @return	Die Liste der Spuren im Universum
 */
Spur **TraffSpot::gibSpurenliste(){
	return spurenliste_;
}

/*!
 * @fn		void TraffSpot::setzeTaktzeit( double taktZeit )
 * @brief	Setzt die Taktzeit, welche bei einem Takt im Universum vergehen soll in s. Es sind Parameter zwischen 0.001 .. 0.1s erlaubt. Diese Zeit entspricht der Zeit, welche ein Mensch hat um eine Entscheidung zu treffen.
 * 
 * @author	Nico Schroeder
 *
 * @throw	Fehler* Einen Fehler, wenn die Taktzeit ausserhalb des Toleranzbereiches liegt.
 */
void TraffSpot::setzeTaktzeit( double taktZeit ){
	if( taktZeit >= 0.001 && taktZeit <= 0.1 ){
		taktZeit_ = taktZeit;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Taktzeit der TraffSpot darf sich nur in den Grenzen zwischen 0.001 und 0.1 s bewegen.");
		throw(fehler);
	}

}

/*!
 * @fn		void TraffSpot::initialisieren( string xmlDatei )
 * @brief	Initialisiert alle Attribute fuer die TraffSpot. Die XML-Datei wird geparst  ... danach kann die TraffSpots gestartet werden (TraffSpot::takt(..))
 * 
 * @author	Nico Schroeder
 *
 * @param	xmlDatei Der Name der XML-Datei
 * @throw	Fehler* Einen Fehler, wenn die Initialisierung nicht vollstaendig abgearbeitet werden konnte
 */
void TraffSpot::initialisieren( string xmlDatei ){
	try{
		clear();
		xmlDatei_ = xmlDatei;
		oeffneXml();
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}


/*!
 * @fn		void TraffSpot::setzeNamen( string TraffSpotName )
 * @brief	Schreibt die Ueberschriften, samt Datum, Uhrzeit und TraffSpotsname in die Statistikdateien
 * 
 * @author	Nico Schroeder
 *
 * @param	traffSpotName Gibt den Namen der Simulation an
 */
void TraffSpot::setzeNamen( string traffSpotName ){
	traffSpotName_ = traffSpotName;
}

/*!
 * @fn		void TraffSpot::erstelleStatistikUeberschriften( string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen )
 * @brief	Schreibt die Ueberschriften, samt Datum, Uhrzeit und TraffSpotsname in die Statistikdateien
 * 
 * @author	Nico Schroeder
 *
 * @param	cvsDateinameFahrzeuge Der Dateiname fuer die Fahrzeugstatistiken
 * @param	cvsDateinameKreuzungen Der Dateiname fuer die Kreuzungsstatistiken
 * @throw	Fehler* Einen Fehler, wenn die Ueberschriften nicht in die Statistikdateien geschrieben werden konnten
 */
void TraffSpot::erstelleStatistikUeberschriften( string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen ){
	string dateiname = cvsDateinameFahrzeuge;
	ofstream cvsDatei;
	time_t zeitstempel;
	tm *aktuelleZeit;
	zeitstempel = time( 0 );
	aktuelleZeit = localtime( &zeitstempel );

	// Datei fuer Fahrzeuge oeffnen
	cvsDatei.open( dateiname.c_str(), ios::app );
	if( !cvsDatei ){
		Fehler *fehler = new Fehler( "Die CVS-Datei fuer Fahrzeuge konnte nicht geoeffnet werden.\n");
		fehler->ergaenzeMeldung( dateiname );
		throw(fehler);
		return;
	}

	// Ueberschriften einfuegen
	cvsDatei << "\"TraffSpot:\";\"" << traffSpotName_ << "\";\"Dauer:\";\"" << vergangeneZeit_ << "\"" << endl;
	cvsDatei << "\"Fahrzeugstatistiken\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Datum:\";\"" << aktuelleZeit->tm_mday << "." << (aktuelleZeit->tm_mon+1) << "." << aktuelleZeit->tm_year+1900 << "\";\"\";\"\"" << endl;
	cvsDatei << "\"Uhrzeit:\";\"" << aktuelleZeit->tm_hour << ":" << aktuelleZeit->tm_min << ":" << aktuelleZeit->tm_sec << "\";\"\";\"\"" << endl;
	cvsDatei << "\"Fahrezug-Id\";\"Typ\";\"Alter in s\";\"Kilometerstand in km\"" << endl;

	// Datei schliessen
	cvsDatei.close();

	dateiname = cvsDateinameKreuzungen;
	// Datei fuer Kreuzungen oeffnen
	cvsDatei.open( dateiname.c_str(), ios::app );
	if( !cvsDatei ){
		Fehler *fehler = new Fehler( "Die CVS-Datei fuer Kreuzungen konnte nicht geoeffnet werden.\n");
		fehler->ergaenzeMeldung( dateiname );
		throw(fehler);
		return;
	}

	// Ueberschriften einfuegen
	cvsDatei << "\"TraffSpot:\";\"" << traffSpotName_ << "\";\"Dauer:\";\"" << vergangeneZeit_ << "\";\"\";\"\"" << endl;
	cvsDatei << "\"Kreuzungsstatistiken\";\"\";\"\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Datum:\";\"" << aktuelleZeit->tm_mday << "." << (aktuelleZeit->tm_mon+1) << "." << aktuelleZeit->tm_year+1900 << "\";\"\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Uhrzeit:\";\"" << aktuelleZeit->tm_hour << ":" << aktuelleZeit->tm_min << ":" << aktuelleZeit->tm_sec << "\";\"\";\"\";\"\";\"\"" << endl;
	
	// Datei schliessen
	cvsDatei.close();
}


/*!
 * @fn		void TraffSpot::exportieren( string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen )
 * @brief	Stoesst den Export der Statistiken an.
 * 
 * @author	Nico Schroeder
 *
 * @param	cvsDateinameFahrzeuge Der Dateiname fuer die Fahrzeugstatistiken
 * @param	cvsDateinameKreuzungen Der Dateiname fuer die Kreuzungsstatistiken
 * @throw	Fehler* Einen Fehler, wenn die Ausgabe der Statistiken in CSV-Dateien nicht erfolgreich war
 */
void TraffSpot::exportieren( string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen ){
	try{
		erstelleStatistikUeberschriften( cvsDateinameFahrzeuge, cvsDateinameKreuzungen );
		kreuzungsliste_->exportieren( cvsDateinameKreuzungen );
		fahrzeugliste_->exportieren( cvsDateinameFahrzeuge );
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		long int TraffSpot::gibAnzahlSpuren( )
 * @brief	Gibt die Anzahl der Spuren
 * 
 * @author	Nico Schroeder
 *
 * @return	Die Anzahl der Spuren in der Liste
 */
long int TraffSpot::gibAnzahlSpuren( ){
	return anzahlSpuren_;
}

/*!
 * @fn		void TraffSpot::oeffneXml( )
 * @brief	Oeffnet und parst die XML-Datei
 * 
 * @author	Nico Schroeder
 *
 * @throw	Fehler* Einen Fehler, wenn die XML-Datei nicht geoeffnet werden kann
 */
void TraffSpot::oeffneXml( ){
	try{
		SzenarioParser *parser = new SzenarioParser( (char *)xmlDatei_.c_str(), &spurenliste_, generatorliste_, kreuzungsliste_, fahrzeugliste_ );
		anzahlSpuren_ = parser->parseXml();
		delete parser;
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		void TraffSpot::clear()
 * @brief	Setzt das Objekt auf die Anfangseinstellungen zurueck
 * 
 * @author	Nico Schroeder
 */
void TraffSpot::clear(){
	// Erstelle die Listen
	if( fahrzeugliste_ != 0 ){
		delete fahrzeugliste_;
	}
	fahrzeugliste_ = new Liste();
	
	if( generatorliste_ != 0 ){
		delete generatorliste_;
	}
	generatorliste_ = new Liste();
	
	if( kreuzungsliste_ != 0 ){
		delete kreuzungsliste_;
	}
	kreuzungsliste_ = new Liste();

	// Initialisiere Defaultwerte fuer weitere private Attribute
	if( spurenliste_ != 0 ){
		delete spurenliste_;
	}
	spurenliste_ = 0;

	anzahlSpuren_ = 0;
	xmlDatei_ = "";
	traffSpotName_ = "unbenannt";	
	taktZeit_ = 0.1;
	vergangeneZeit_ = 0;
}