/*!
 * @file  TraffSpot.h
 * @brief Definition der Klassen TraffSpot.
 */ 

#ifndef __TRAFF_SPOT_
#define __TRAFF_SPOT_

#include <iostream>
#include <fstream>
#include <string>
#include <time.h>
using namespace std;

#include "Fehler.h"
#include "Liste.h"
#include "Spur.h"
#include "xml/SzenarioParser.h"

/*!
 * @class TraffSpot
 * @brief Das Kernstueck des Projektes. Diese Klasse steuert die komplette TraffSpot.
 */ 
class TraffSpot
{
	public:
		TraffSpot();

		void initialisieren( string xmlDatei );
		void setzeNamen( string TraffSpotName );
		void takt();
		void setzeTaktzeit( double taktZeit );
		void beenden();
		void exportieren( string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen );

		Liste *gibFahrzeugliste();
		Liste *gibGeneratorliste();
		Liste *gibKreuzungsliste();
		Spur **gibSpurenliste();
		long int gibAnzahlSpuren();

	private:
		void clear();
		void oeffneXml( );
		void erstelleStatistikUeberschriften(string cvsDateinameFahrzeuge, string cvsDateinameKreuzungen);

		Liste *fahrzeugliste_;	//!< Die Liste aller an der Simulation beteiligter Fahrzeuge
		Liste *generatorliste_;	//!< Die Liste der Generatoren, welche in der Simulation kontinuierlich Fahrzeuge erstellen
		Liste *kreuzungsliste_;	//!< Die Liste aller Kreuzungen, an denen eine Ampelschaltung erfolgen muss
		Spur **spurenliste_;	//!< Ein Array in dem alle Spuren registriert sind. Ausserdem sind die Spuren untereinander zu einem Graphen verbunden. In den Knoten befinden sich Spuren.
		long int anzahlSpuren_;	//!< Die Anzahl der Spuren, damit ein Zugriff auf das Array gewaehrt werden kann
		string xmlDatei_;		//!< Dateiname zu der XML-Datei
		string traffSpotName_;	//!< Ein Identifier, um z.B. Logfiles von einander unterscheiden zu koennen
		double taktZeit_;		//!< Der Zeitabstand zwischen zwei Takten. ACHTUNG: Virtueller Zeitabstand nicht reeler
		double vergangeneZeit_; //!< Die Zeit in s, welche die Simulation schon gelaufen ist.
};

#endif