/*! 
 * @file GLWireframe.h
 * @brief Definition der Klasse GLWireframe.
 */

#ifndef __GLWIREFRAME__
#define __GLWIREFRAME__

#include <qgl.h>


#include "../Fahrzeug.h"
#include "../Kreuzung.h"
#include "../Konstanten.h"
#include "../Liste.h"
#include "../Spur.h"


/*! 
 * @class GLWireframe
 * @brief Widget zur Darstellung der Objekte in OpenGL.
 * @author Nico Schroeder
 */
class GLWireframe : public QGLWidget
{
    Q_OBJECT

public:
 GLWireframe( QWidget* parent = 0, const char* name = 0, QGLWidget *share = 0, WFlags f = 0, Spur** spurenliste = 0, long int anzahlSpuren = 0, Liste *kreuzungsliste = 0 );
 GLWireframe( QWidget* parent , const char* name    , QGLWidget *share , WFlags f    , 
           double xRotD, double yRotD, double zRotD, double xKameraD, double yKameraD, double zKameraD, Spur** spurenliste, long int anzahlSpuren, Liste *kreuzungsliste  );
 ~GLWireframe();

 double gibXRot();
 double gibYRot();
 double gibZRot();
 
 double gibXKamera();
 double gibYKamera();
 double gibZKamera(); 

 void setzeFahrzeugliste( Liste *fahrzeugliste );
 void setzeSpurenliste(Spur** spurenliste, long int anzahlSpuren);
 void setzeKreuzungsliste(Liste *kreuzungsliste);

 void zeichne();
 
public slots:
    void setzeXRotation( double grad );
    void setzeYRotation( double grad );
    void setzeZRotation( double grad );

 void setzeRotationImpulse( double x, double y, double z );
 void setzeTranslationImpulse( double x, double y);
 void setzeZoom( double aenderung );
 
 void setzeMaussteuerung( int maussteuerung );
 
 void ladeWerkseinstellungen();

protected:
 void initializeGL();
 void paintGL();
 void resizeGL( int breite, int hoehe );
 void transform();
 
 void clear();
 void ladeStandardwerte();
 double gibMaximaleAusdehnung();
 void erzeugeParameter();
 
 void mousePressEvent( QMouseEvent *event );
    void mouseReleaseEvent( QMouseEvent *event );
 void mouseMoveEvent( QMouseEvent *event );
 void wheelEvent( QWheelEvent *event );
 
 void setMaterialColor3d(Koordinaten farbe);
 void erzeugeZeichenliste();
 void zeichneSpur(Spur *spur);
 void zeichneHaltelinie(Spur *spur);
 void zeichneSpurverbindung(Spur *vorgaenger, Spur *nachfolger);
 void zeichneFahrzeug( Fahrzeug *fahrzeug );
 void zeichneKreuzung( Kreuzung *kreuzung );
 
private:

 GLuint list_;      //!< Liste aller nicht veraenderlichen Objekte anlegen
 long int anzahlSpuren_;    //!< Die Anzahl der darzustellenden Spuren
 double maximaleKoordinaten_[3];     //!< Speichert die maximale x, y und z-Koordinate der darzustellenden Objekte
 double minimaleKoordinaten_[3];  //!< Speichert die minimalen x, y und z-Koordinate der darzustellenden Objekte
 double verschiebungsKoordinaten_[3];//!< Gibt an wie weit die Objekte aus ihrem urspruenglichen Koordinatensystem verschoben werden mussen um den Punkt(0|0|0) ungefaehr im Schwerpunkt der darzustellenden Objekte zu bekommen

    GLfloat xRot_;       //!< Gibt die Drehung um die x-Achse an
 GLfloat yRot_;       //!< Gibt die Drehung um die y-Achse an
 GLfloat zRot_;       //!< Gibt die Drehung um die z-Achse an
	double punktKamera_[3];      		//!< Spezifiziert den Punkt, an dem die Kamera liegt
	double punktReferenz_[3];    		//!< Zu diesem Punkt schaut die Kamera
	double vectorKameraOben_[3]; 		//!< Dieser Vector zeigt an, wo oben an der Kamera ist
   	QPoint oldPos_;						//!< Hilfsvariable zum Speichern der alten Mausposition
	
	GLfloat standardxRot_; 				//!< Gibt den Standardwert fuer die Drehung um die x-Achse an
	GLfloat standardyRot_;				//!< Gibt den Standardwert fuer die Drehung um die y-Achse an
	GLfloat standardzRot_; 				//!< Gibt den Standardwert fuer die Drehung um die z-Achse an
	double standardpunktKamera_[3];		//!< Spezifiziert den Standardwert fuer den Punkt, an dem die Kamera liegt
	double standardpunktReferenz_[3];	//!< Zu diesem Punkt schaut die Kamera (Standardwert)
	double standardvectorKameraOben_[3];//!< Dieser Vector zeigt an, wo oben an der Kamera ist (Standardwert)

	Liste *fahrzeugliste_;	//!< Die Liste aller an der Simulation beteiligter Fahrzeuge
	Liste *generatorliste_;	//!< Die Liste der Generatoren, welche in der Simulation kontinuierlich Fahrzeuge erstellen
	Liste *kreuzungsliste_;	//!< Die Liste aller Kreuzungen, an denen eine Ampelschaltung erfolgen muss
	Spur **spurenliste_;	//!< Ein Array in dem alle Spuren registriert sind. Ausserdem sind die Spuren untereinander zu einem Graphen verbunden. In den Knoten befinden sich Spuren.
	
	int maussteuerung_;		//!< Gibt an, welche Maussteuerung verwendet werden soll.
};


#endif
