/*!
 * @file  StructureParser.cpp
 * @brief Implementierung der Klassen StructureParser, StructureCounter und StructureGetter.
 */

#include "StructureParser.h"

#include "../Koordinaten.h"
#include "../Konstanten.h"
#include "../Spur.h"
#include "../Kreuzung.h"
#include "../Generator.h"

#include <stdio.h>
#include <stdlib.h>
#include <qstring.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		StructureParser::StructureParser()
 * @brief	Standardkonstruktor der Klasse StructureParser.
 *
 * @author	Andreas Richter
 */
StructureParser::StructureParser()
{
	// please do not use
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		StructureParser::StructureParser( Spur **spurenArray, long int *spurenPosition, Liste *generatorListe, Liste *kreuzungsListe, Liste *fahrzeugeListe )
 * @brief	Ueberladener Konstruktor der Klasse StructureParser, der die Zeiger auf die Listen mit abspeichert.
 *
 * @author	Andreas Richter
 *
 * @param   spurenArray Zeiger auf das erste Element des SpurenArrays.
 * @param   spurenPosition Zeiger auf Hilfsarray, welches die Position der Spuren im spurenArray_ speichert.
 * @param   generatorListe Zeiger auf die Liste der Fahrzeuggeneratoren.
 * @param	kreuzungsListe Zeiger auf die Liste der Kreuzungen.
 * @param	fahrzeugeListe Zeiger auf die Liste der Fahrzeuge.
 */
StructureParser::StructureParser( Spur **spurenArray, long int *spurenPosition, Liste *generatorListe, Liste *kreuzungsListe, Liste *fahrzeugeListe )
{
	   spurenArray_ = spurenArray;
	spurenPosition_ = spurenPosition;
	generatorListe_ = generatorListe;		
	kreuzungsListe_ = kreuzungsListe;
	fahrzeugeListe_ = fahrzeugeListe;
	durchlauf_ = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		bool StructureParser::startElement( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr )
 * @brief	Wird aufgerufen, wenn ein Startelement-Tag gelesen wird.
 *
 * @author	Andreas Richter
 *
 * @param   qName Name des Element-Tags
 * @param   qAttr Verweis auf die Attribute des Element-Tags
 *
 * @return  TRUE Gibt TRUE zurueck.
 *
 * @throw   fehler Wirft einen Fehler, wenn ein Attribut leer ist.
 */
bool StructureParser::startElement( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr )
{
	//--- Element Spur gelesen ------------------------------------------------
	if( qName == "spur" )
	{
		int			ausrichtung = 0, richtung = 0, vMax = 0;
		long int	id = 0;
		double		laenge = 0, x = 0, y = 0, z = 0;
		Koordinaten *coords = new Koordinaten();
		
		idSpur = "unbekannt";
		
		for( int i = 0; i < qAttr.count(); i++)
		{   // ID
			if( qAttr.localName( i ) == "id" )
			{
				id = atoi( (const char*)qAttr.value( i ) );
				idSpur = (const char*)qAttr.value( i );
				if( id == 0 )
				{
					Fehler *fehler = new Fehler("Eine SpurenID ist leer. Bitte berprfen Sie die XML-Datei.");
					throw( fehler );
				}
			}
			
			// Laenge
			if( qAttr.localName( i ) == "laenge" )
			{
				laenge = atof( (const char*)qAttr.value( i ) )/10;
				if( laenge == 0 )
				{
					string text = "Die Lnge der Spur #"; text += idSpur; text += " ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// Ausrichtung
			if( qAttr.localName( i ) == "ausrichtung" )
			{
				if( strcmp((const char*)qAttr.value( i ),"HOR") == 0 )
					ausrichtung = HOR;
				else
				{
					if( strcmp((const char*)qAttr.value( i ),"VER") == 0 )
							ausrichtung = VER;
					else
					{
						string text = "Die Ausrichtung der Spur #"; text += idSpur; text += " hat keinen gltigen Wert. Bitte berprfen Sie die XML-Datei.";
						Fehler *fehler = new Fehler( text );
						throw( fehler );
					}
				}
			}
			// Richtung
			if( qAttr.localName( i ) == "richtung" )
			{
				if ( strcmp( (const char*)qAttr.value( i ),"LTOR") == 0 || strcmp( (const char*)qAttr.value( i ), "UP") == 0 )
					richtung = LTOR;
				else
				{
					if ( strcmp( (const char*)qAttr.value( i ),"RTOL") == 0 || strcmp( (const char*)qAttr.value( i ), "DOWN") == 0 )
							richtung = RTOL;
					else
					{
						string text = "Die Richtung der Spur #"; text += idSpur; text += " hat keinen gltigen Wert. Bitte berprfen Sie die XML-Datei.";
						Fehler *fehler = new Fehler( text );
						throw( fehler );
					}
				}
			}
			
			// Koordinaten
			if( qAttr.localName( i ) == "x" )
				x = atof( (const char*)qAttr.value( i ) );
				
			if( qAttr.localName( i ) == "y" )
				y = atof( (const char*)qAttr.value( i ) );
			
			if( qAttr.localName( i ) == "z" )
				z = atof( (const char*)qAttr.value( i ) );
			
			// maximale Geschwindigkeit
			if( qAttr.localName( i ) == "maximaleGeschwindigkeit" )
			{
				vMax = atoi( (const char*)qAttr.value( i ) );
				if( vMax == 0 )
				{
					string text = "Die maximale Geschindigkeit der Spur #"; text += idSpur; text += " ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
		}
		
		coords->x = x;
		coords->y = y;
		coords->z = z;
		
		// Spur anlegen und mit Attributen fuellen
		spurenArray_[durchlauf_] = new Spur( id, laenge, vMax );
		
		spurenArray_[durchlauf_]->setzeKoordinaten( *coords );
		spurenArray_[durchlauf_]->setzeRichtung( richtung );
		spurenArray_[durchlauf_]->setzeAusrichtung( ausrichtung );
		
		// Hilfsarray mit Position im Spurenarray befuellen
		spurenPosition_[id] = durchlauf_;
		
		// in der Liste einen Schritt weiter gehen
		durchlauf_++;
	}



	//--- Element Nachfolger gelesen ------------------------------------------
	if( qName == "nachfolger" )
	{   
		long int id = 0;
		const char	*idNachfolger = "unbekannt";
		int		 richtung = 0;
		double   wahrscheinlichkeit = 0;
		
		for( int i = 0; i < qAttr.count(); i++)
		{    // ID
			if( qAttr.localName( i ) == "id" )
			{
				id = atoi( (const char*)qAttr.value( i ) );
				idNachfolger = (const char*)qAttr.value( i );
				if( id == 0 )
				{
					string text = "Die ID eines Nachfolgers ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
				
			
			// Richtung
			if( qAttr.localName( i ) == "richtung" )
			{
				if( strcmp((const char*)qAttr.value( i ),"LINKS") == 0 )
					richtung = LINKS;
				else
				{
					if( strcmp((const char*)qAttr.value( i ),"MITTE") == 0 )
						richtung = MITTE;
					else
					{
						if( strcmp((const char*)qAttr.value( i ),"RECHTS") == 0 )
							richtung = RECHTS;
						else
						{
							string text = "Die Abbiegerichtung eines Nachfolgers ist leer. Bitte berprfen Sie die XML-Datei.";
							Fehler *fehler = new Fehler( text );
							throw( fehler );
						}
					}
				}
			}

			
			// Abbiegewahrscheinlichkeit
			if( qAttr.localName( i ) == "wahrscheinlichkeit" )
			{
				wahrscheinlichkeit = atof( (const char*)qAttr.value( i ) );
				if( wahrscheinlichkeit == 0 )
				{
					string text = "Die Abbiegewahrscheinlichkeit eines Nachfolgers ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
		}
		
		spurenArray_[durchlauf_-1]->setzeNaechstesVerbindungselement( richtung, id, wahrscheinlichkeit);
	}
	


	//--- Element Generator gelesen -------------------------------------------
	if( qName == "generator" )
	{
		for( int i = 0; i < qAttr.count(); i++)
		{  	// ID des Generators
			if( qAttr.localName( i ) == "id" )
			{
				generator_generatorId = atoi( (const char*)qAttr.value( i ) );
				idGenerator = (const char*)qAttr.value( i );
				if( idGenerator == 0 )
				{
					Fehler *fehler = new Fehler("Eine GeneratorID ist leer. Bitte berprfen Sie die XML-Datei.");
					throw( fehler );
				}
			}
			
			// ID der Spur
			if( qAttr.localName( i ) == "spurID" )
			{
				generator_spurId = atoi( (const char*)qAttr.value( i ) );
				if( generator_spurId == 0 )
				{
					string text = "Die SpurenID des Generators #"; text += idGenerator; text += " ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}

			// Anzahl der Fahrzeuge
			if( qAttr.localName( i ) == "fahrzeugAnzahl" )
			{
				generator_anzahl = atoi( (const char*)qAttr.value( i ) );
				if( generator_anzahl == 0 )
				{
					string text = "Die Anzahl der zu generierenden Fahrzeuge des Generators #"; text += idGenerator; text += " ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// Zeitintervall
			if( qAttr.localName( i ) == "erstellzeit" )
			{
				generator_zeit = atof( (const char*)qAttr.value( i ) );
				if( generator_zeit == 0 )
				{
					string text = "Die Erstellzeit des Generators #"; text += idGenerator; text += " ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );

				}
			}		
		}

		// Generator erstellen und mit Kenndaten fuellen
		generator = new Generator( spurenArray_[ spurenPosition_[generator_spurId] ], generator_generatorId );
		
		generator->setzeFahrzeugliste( fahrzeugeListe_ );
		generator->setzeAnzahlFahrzeuge( generator_anzahl );
		generator->setzeTaktZeit( generator_zeit );
		
		generatorListe_->datenHinzufuegen( generator );			
	}
			

			
			
	//--- Element Fahrzeug gelesen --------------------------------------------
	if( qName == "fahrzeug" )
	{
		int			typ = 0, ttl = 0;
		double		laenge = 0, aPos = 0, aNeg = 0, vMax = 0, r = 0, g = 0, b = 0;
		Koordinaten *color  = new Koordinaten();
		
		for( int i = 0; i < qAttr.count(); i++)
		{   // Typ
			if( qAttr.localName( i ) == "typ" )
			{
				if( strcmp((const char*)qAttr.value( i ),"PKW") == 0 )
					typ = PKW;
				else
				{
					if( strcmp((const char*)qAttr.value( i ),"BUS") == 0 )
						typ = BUS;
					else
					{
						if( strcmp((const char*)qAttr.value( i ),"LKW") == 0 )
							typ = LKW;
						else
						{
							if( strcmp((const char*)qAttr.value( i ),"BIKE") == 0 )
								typ = BIKE;
							else
							{
								string text = "Ein Fahrzeug eines Generators hat keinen gltigen Typ. Bitte berprfen Sie die XML-Datei.";
								Fehler *fehler = new Fehler( text );
								throw( fehler );
							}
						}
					}
				}
			}

			// Laenge
			if( qAttr.localName( i ) == "laenge" )
			{
				laenge = atof( (const char*)qAttr.value( i ) );
				if( laenge == 0 )
				{
					string text = "Die Fahrzeuglnge eines Generators ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// Lebensdauer
			if( qAttr.localName( i ) == "lebensdauer" )
			{
				ttl = atoi( (const char*)qAttr.value( i ) );
				if( ttl == 0 )
				{
					string text = "Die Fahrzeuglebensdauer eines Generators ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// positive Beschleunigung
			if( qAttr.localName( i ) == "beschleunigungPositiv" )
			{
				aPos = atof( (const char*)qAttr.value( i ) );
				if( aPos == 0 )
				{
					string text = "Die Fahrzeugbeschleunigung eines Generators ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// negative Beschleunigung
			if( qAttr.localName( i ) == "beschleunigungNegativ" )
			{
				aNeg = atof( (const char*)qAttr.value( i ) );
				if( aNeg == 0 )
				{
					string text = "Die Fahrzeugverzgerung eines Generators ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
				if( aNeg > 0 )
				{
					string text = "Die Fahrzeugverzgerung eines Generators muss negativ sein. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// maximale Geschwindigkeit
			if( qAttr.localName( i ) == "hoechstgeschwindigkeit" )
			{
				vMax = atoi( (const char*)qAttr.value( i ) );
				if( vMax == 0 )
				{
					string text = "Die Fahrzeughchstgeschwindigkeit eines Generators ist leer. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			// Farbe
			if( qAttr.localName( i ) == "rot" )
			{
				r = atof( (const char*)qAttr.value( i ) );
				if( r > 255 )
				{
					string text = "Der Rotanteil der Fahrzeugfarbe eines Generators grer als 255. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			if( qAttr.localName( i ) == "gruen" )
			{
				g = atof( (const char*)qAttr.value( i ) );
				if( g > 255 )
				{
					string text = "Der Grnanteil der Fahrzeugfarbe eines Generators grer als 255. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			if( qAttr.localName( i ) == "blau" )
			{
				b = atof( (const char*)qAttr.value( i ) );
				if( b > 255 )
				{
					string text = "Der Blauanteil der Fahrzeugfarbe eines Generators grer als 255. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
			color->x = r;
			color->y = g;
			color->z = b;
		}

		// Fahrzeugeigenschaften dem aktuellen Generator uebergeben
		generator->setzeFahrzeugeigenschaften( typ, aPos, aNeg, ttl, laenge, *color, vMax );
	}
	

	
	//--- Element Farhrtenbucheintrag gelesen ------------------------------------------------
	if( qName == "fahrtenbucheintrag" )
	{
		int richtung = 0;
		
		for( int i = 0; i < qAttr.count(); i++)
		{  
			// Richtung
			if( qAttr.localName( i ) == "richtung" )
			{
				if( strcmp((const char*)qAttr.value( i ),"LINKS") == 0 )
					richtung = LINKS;
				else 
				{
					if( strcmp((const char*)qAttr.value( i ),"MITTE") == 0 )
						richtung = MITTE;
					else
					{
						if( strcmp((const char*)qAttr.value( i ),"RECHTS") == 0 )
							richtung = RECHTS;
						else
						{
							Fehler *fehler = new Fehler( "Ein Fahrtenbucheintrag eines Generators ist nicht korrekt. Bitte berprfen Sie die XML-Datei.");
							fehler->ergaenzeMeldung( (char*) generator->gibId() );
							throw(fehler);
						}
					}
				}
			}
		}
		
		// Fahrtenbuch eintragen
		generator->wegpunktHinzufuegen( richtung );
	}
			
		
	//--- Element Kreuzung gelesen ------------------------------------------------
	if( qName == "kreuzung" )
	{
		long int id = 0, nL = 0, nM = 0, nR = 0, oL = 0, oM = 0, oR = 0, sL = 0, sM = 0, sR = 0, wL = 0, wM = 0, wR = 0;
		const char *idKreuzung = "unbekannt";
		int		 ampelAlgo = 1;

		for( int i = 0; i < qAttr.count(); i++)
		{   // Norden
			if( qAttr.localName( i ) == "nordenLinks" )
				nL = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "nordenMitte" )
				nM = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "nordenRechts" )
				nR = atoi( (const char*)qAttr.value( i ) );

			// Osten
			if( qAttr.localName( i ) == "ostenLinks" )
				oL = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "ostenMitte" )
				oM = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "ostenRechts" )
				oR = atoi( (const char*)qAttr.value( i ) );

			// Sueden
			if( qAttr.localName( i ) == "suedenLinks" )
				sL = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "suedenMitte" )
				sM = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "suedenRechts" )
				sR = atoi( (const char*)qAttr.value( i ) );

			// Westen
			if( qAttr.localName( i ) == "westenLinks" )
				wL = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "westenMitte" )
				wM = atoi( (const char*)qAttr.value( i ) );
			if( qAttr.localName( i ) == "westenRechts" )
				wR = atoi( (const char*)qAttr.value( i ) );
		
			// ID der Spur
			if( qAttr.localName( i ) == "id" )
			{
				id = atoi( (const char*)qAttr.value( i ) );
				idKreuzung = (const char*)qAttr.value( i );
				if( id == 0 )
				{
					Fehler *fehler = new Fehler("Eine KreuzungsID ist leer. Bitte berprfen Sie die XML-Datei.");
					throw( fehler );
				}
			}

			// Ampelalgorithmus
			if( qAttr.localName( i ) == "ampelalgorithmus")
			{
				ampelAlgo = atoi( (const char*)qAttr.value( i ) );
				if( ampelAlgo == 0 )
				{
					string text = "Der Ampelalgorithmus der Kreuzung #"; text += idKreuzung; text += " ist nicht gesetzt. Bitte berprfen Sie die XML-Datei.";
					Fehler *fehler = new Fehler( text );
					throw( fehler );
				}
			}
			
		}

		// Kreuzung erstellen und mit Kenndaten fuellen
		Kreuzung   *kreuzung = new Kreuzung( id );

		if( nL != 0 )
			kreuzung->setzeSpur( NORD, LINKS, spurenArray_[ spurenPosition_[nL] ] );
		if( nM != 0 )
			kreuzung->setzeSpur( NORD, MITTE, spurenArray_[ spurenPosition_[nM] ] );
		if( nR != 0 )
			kreuzung->setzeSpur( NORD, RECHTS, spurenArray_[ spurenPosition_[nR] ] );
		if( oL != 0 )
			kreuzung->setzeSpur( OST, LINKS, spurenArray_[ spurenPosition_[oL] ] );
		if( oM != 0 )
			kreuzung->setzeSpur( OST, MITTE, spurenArray_[ spurenPosition_[oM] ] );
		if( oR != 0 )
			kreuzung->setzeSpur( OST, RECHTS, spurenArray_[ spurenPosition_[oR] ] );
		if( sL != 0 )
			kreuzung->setzeSpur( SUED, LINKS, spurenArray_[ spurenPosition_[sL] ] );
		if( sM != 0 )
			kreuzung->setzeSpur( SUED, MITTE, spurenArray_[ spurenPosition_[sM] ] );
		if( sR != 0 )
			kreuzung->setzeSpur( SUED, RECHTS, spurenArray_[ spurenPosition_[sR] ] );
		if( wL != 0 )
			kreuzung->setzeSpur( WEST, LINKS, spurenArray_[ spurenPosition_[wL] ] );
		if( wM != 0 )
			kreuzung->setzeSpur( WEST, MITTE, spurenArray_[ spurenPosition_[wM] ] );
		if( wR != 0 )
			kreuzung->setzeSpur( WEST, RECHTS, spurenArray_[ spurenPosition_[wR] ] );

		try
		{
			kreuzung->setzeAmpelalgorithmus( ampelAlgo );
		}
		catch(Fehler f)
		{
			string text = "Der Ampelalgorithmus der Kreuzung #"; text += idKreuzung; text += " hat keinen gltigen Wert. Bitte berprfen Sie die XML-Datei.";
			Fehler *fehler = new Fehler( text );
			throw( fehler );
		}
		
		kreuzungsListe_->datenHinzufuegen( kreuzung );
	}

	return TRUE;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++






//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		StructureCounter::StructureCounter()
 * @brief	Standardkonstruktor der Klasse StructureCounter.
 *
 * @author	Andreas Richter
 */
StructureCounter::StructureCounter()
{
	countSpuren_ = 0;
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		bool StructureCounter::endElement( const QString&, const QString&, const QString &qName )
 * @brief	Wird aufgerufen, wenn ein Endelement-Tag gelesen wird und zaehlt jedes Spurenelement.
 *
 * @author	Andreas Richter
 *
 * @param   qName Name des Element-Tags
 *
 * @return  Gibt TRUE zurueck.
 */
bool StructureCounter::endElement( const QString&, const QString&, const QString &qName )
{
	if( qName == "spur" )
		countSpuren_ += 1;
	
	return TRUE;
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		int StructureCounter::gibAnzahlSpuren()
 * @brief	Gibt die Anzahl der gelesenen Spuren zurueck.
 *
 * @author	Andreas Richter
 *
 * @return  Gibt die Anzahl der Spuren in der XML-Datei zurueck.
 */
int StructureCounter::gibAnzahlSpuren()
{
	return countSpuren_;
}
//------------------------------------------------------------------------------------------------------------------



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		StructureGetter::StructureGetter()
 * @brief	Standardkonstruktor der Klasse StructureGetter.
 *
 * @author	Andreas Richter
 */
StructureGetter::StructureGetter()
{
	maxID_ = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		bool StructureGetter::startElement( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr )
 * @brief	Wird aufgerufen, wenn ein Startelement-Tag gelesen wird und wertet die SpurenID aus.
 *
 * @author	Andreas Richter
 *
 * @param   qName Name des Element-Tags
 * @param   qAttr Verweis auf die Attribute des Element-Tags
 *
 * @return  Gibt TRUE zurueck.
 */
bool StructureGetter::startElement( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr )
{
	if( qName == "spur" )
	{
		long int	id = 0;
		
		for( int i = 0; i < qAttr.count(); i++)
		{   // ID
			if( qAttr.localName( i ) == "id" )
				id = atol( (const char*)qAttr.value( i ) );
		}
		
		// ggf. neue maxID setzen
		if( id > maxID_ )
			maxID_ = id;
	}
	
	return TRUE;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*!
 * @fn		long int StructureGetter::gibMaxId()
 * @brief	Gibt die groesste SpurenID zurueck.
 *
 * @author	Andreas Richter
 *
 * @return  Gibt groesste SpurenID zurueck.
 */
long int StructureGetter::gibMaxId()
{
	return maxID_;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++