/*!
 * @file  StructureParser.h
 * @brief Definition der Klassen StructureParser, StructureCounter und StructureGetter.
 */

#ifndef STRUCTURE_PARSER_H
#define STRUCTURE_PARSER_H   

#include "../Spur.h"
#include "../Liste.h"
#include "../Generator.h"

#include <qxml.h>

class QString;

/*!
 * @class StructureParser
 * @brief Parst eine als XML hinterlegte Strassenkarte und schreibt entsprechende Daten in Listen und Arrays.
 */
class StructureParser : public QXmlDefaultHandler
{
public:
			StructureParser ( Spur **spurenArray, long int *spurenPosition, Liste *generatorListe, Liste *kreuzungsListe, Liste *fahrzeugeListe );
    bool	startElement	( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr );

private:
		StructureParser();

	Spur **spurenArray_;		//!< Zeiger auf das erste Element des Spuren
	long int *spurenPosition_;  //!< Hilfsarray zum Wiederfinden der Spuren im Spurenarray
	Liste *generatorListe_;		//!< Zeiger auf die Liste der Fahrzeuggeneratoren
	Liste *kreuzungsListe_;		//!< Zeiger auf die Liste der Kreuzungen
	Liste *fahrzeugeListe_;		//!< Zeiger auf die Liste der Fahrzeuge
	int    durchlauf_;			//!< Durchlaufvariable, wird pro Spuren-Endelement hochgezaehlt
	int		 generator_anzahl;		//!< (Fuer Generator gueltig) Anzahl der zu generierenden Fahrzeuge
	long int generator_spurId;		//!< (Fuer Generator gueltig) ID der angeschlossenen Spur
	long int generator_generatorId; //!< (Fuer Generator gueltig) ID des Generators
	double	 generator_zeit;		//!< (Fuer Generator gueltig) Intervallzeit des Generators
	Generator    *generator;		//!< (Fuer Generator gueltig) Zeiger auf den aktuellen Generator
	const char	 *idGenerator;		//!< (Fuer Generator gueltig) ID des Generator als String
	const char	 *idSpur;			//!< (Fuer Spur gueltig) ID der Spur als String
	
};

/*!
 * @class StructureCounter
 * @brief Zaehlt Elemente in einer XML-Datei.
 */
class StructureCounter : public QXmlDefaultHandler
{
public:
		 StructureCounter();
    bool endElement( const QString&, const QString&, const QString &qName );
	int	 gibAnzahlSpuren();

private:
	int countSpuren_;		//!< Anzahl der gelesenen Spuren	
};

/*!
 * @class StructureGetter
 * @brief Ermittelt die groesste SpurenID in der XML-Datei.
 */
class StructureGetter : public QXmlDefaultHandler
{
public:
			 StructureGetter();
    bool     startElement( const QString&, const QString&, const QString &qName, const QXmlAttributes &qAttr );
	long int gibMaxId();
	
private:
	long int maxID_;		//!< groesste benutzte SpurenID
};

#endif